package com.jiuqi.office.excel;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.jiuqi.grid.CellField;
import com.jiuqi.grid.CurrencyCellProperty;
import com.jiuqi.grid.GridCell;
import com.jiuqi.grid.GridData;
import com.jiuqi.grid.GridFieldList;
import com.jiuqi.grid.NumberCellProperty;
import com.jiuqi.office.excel.HSSFHelper;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.poi.hssf.usermodel.HSSFRichTextString;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hssf.util.CellRangeAddress;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.RichTextString;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFRichTextString;

public final class WorksheetWriter {
  private Workbook workBook;
  
  private Sheet sheet;
  
  private GridData gridData;
  
  private String title;
  
  private boolean autoAdjust;
  
  private boolean addTitle;
  
  private Font titleFont;
  
  private HashMap cellStyleHashMap;
  
  private int transparentColor;
  
  private boolean doNotCheckNumberIsNull;
  
  private Map HSSFFontMap;
  
  private List sheets;
  
  private static int MAX_COLUMN = 255;
  
  private String preSheetName;
  
  public WorksheetWriter(HSSFWorkbook workBook, HSSFSheet sheet, GridData gridData, HashMap cellStyleHashMap) {
    this.sheets = new ArrayList();
    this.workBook = (Workbook)workBook;
    this.sheet = (Sheet)sheet;
    this.sheets.add(sheet);
    this.gridData = gridData;
    this.cellStyleHashMap = cellStyleHashMap;
    this.HSSFFontMap = new HashMap<Object, Object>();
    this.transparentColor = -16777201;
  }
  
  public WorksheetWriter(Workbook workBook, Sheet sheet, GridData gridData, HashMap cellStyleHashMap) {
    this.sheets = new ArrayList();
    this.workBook = workBook;
    this.sheet = sheet;
    this.sheets.add(sheet);
    this.gridData = gridData;
    this.cellStyleHashMap = cellStyleHashMap;
    this.HSSFFontMap = new HashMap<Object, Object>();
    this.transparentColor = -16777201;
  }
  
  public String getPreSheetName() {
    return this.preSheetName;
  }
  
  public void setPreSheetName(String preSheetName) {
    this.preSheetName = preSheetName;
  }
  
  public void setTitle(String title) {
    this.title = title;
  }
  
  private void setDefaultTitleFont() {
    this.titleFont = this.workBook.createFont();
    this.titleFont.setFontName("宋体");
    this.titleFont.setBoldweight((short)700);
    this.titleFont.setFontHeightInPoints((short)12);
  }
  
  public void setAutoAdjust(boolean autoAdjust) {
    this.autoAdjust = autoAdjust;
  }
  
  public void setAddTitle(boolean addTitle) {
    this.addTitle = addTitle;
  }
  
  public void setTransparentColor(int transparentColor) {
    this.transparentColor = transparentColor;
  }
  
  public boolean isDoNotCheckNumberIsNull() {
    return this.doNotCheckNumberIsNull;
  }
  
  public void setDoNotCheckNumberIsNull(boolean doNotCheckNumberIsNull) {
    this.doNotCheckNumberIsNull = doNotCheckNumberIsNull;
  }
  
  public void writeWorkSheet() {
    addSheets();
    addTitle();
    writeContent();
    adjustHW();
    mergeCells();
    hideRowCol();
  }
  
  private Sheet getSheet(int index) {
    return this.sheets.get(index);
  }
  
  private int getSheetIndex(int col) {
    return col / MAX_COLUMN;
  }
  
  private int getCol(int col) {
    return col % MAX_COLUMN;
  }
  
  private void addSheets() {
    if (this.gridData.getColCount() > MAX_COLUMN) {
      int extraSheetsNum = (this.gridData.getColCount() + MAX_COLUMN - 1) / 
        MAX_COLUMN - 1;
      for (int i = 0; i < extraSheetsNum; i++)
        this.sheets.add(this.workBook.createSheet(String.valueOf(this.preSheetName) + (i + 1))); 
    } 
  }
  
  private void addTitle() {
    if (this.addTitle) {
      setDefaultTitleFont();
      RichTextString richText = getRichTextString(this.title, this.workBook);
      for (int i = 0; i < this.sheets.size(); i++) {
        Cell cell = getSheet(i).createRow(0).createCell(0);
        if (richText != null)
          cell.setCellValue(richText); 
        CellStyle titleCellStyle = this.workBook.createCellStyle();
        titleCellStyle.setAlignment((short)2);
        titleCellStyle.setFont(this.titleFont);
        cell.setCellStyle(titleCellStyle);
        getSheet(i).addMergedRegion(
            (CellRangeAddress)new CellRangeAddress(0, 0, 0, Math.min(this.gridData
                .getColCount() - 2, MAX_COLUMN)));
      } 
    } 
  }
  
  private void writeContent() {
    if (this.gridData == null)
      throw new NullPointerException("表格数据不能为空"); 
    int rowCount = this.gridData.getRowCount();
    int colCount = this.gridData.getColCount();
    for (int row = 1; row < rowCount; row++) {
      for (int col = 1; col < colCount; col++) {
        GridCell gridCell = this.gridData.getCell(col, row);
        writeCell(gridCell);
      } 
    } 
  }
  
  private void writeCell(GridCell gridCell) {
    int rowNum = gridCell.getRowNum() - 1;
    int colNum = gridCell.getColNum() - 1;
    if (this.addTitle)
      rowNum++; 
    Row row = (getSheet(getSheetIndex(colNum)).getRow(rowNum) == null) ? getSheet(
        getSheetIndex(colNum)).createRow(rowNum) : 
      getSheet(getSheetIndex(colNum)).getRow(rowNum);
    Cell cell = row.createCell(getCol(colNum));
    try {
      String strValue, str1;
      Date date;
      String cellData, linkInfoStr;
      RichTextString richTextString;
      CellStyle cellStyle = buildCellStyle(gridCell);
      cell.setCellStyle(cellStyle);
      switch (gridCell.getDataType()) {
        case 0:
        case 1:
        case 4:
          strValue = gridCell.getCellData();
          richTextString = getRichTextString(
              strValue, this.workBook);
          if (richTextString != null)
            cell.setCellValue(richTextString); 
          break;
        case 2:
          str1 = gridCell.getCellData();
          if (str1 != null && !str1.equals("")) {
            double value = gridCell.getFloat();
            cell.setCellValue(value);
            if (gridCell.getShowText().endsWith("‰")) {
              NumberCellProperty ncp = new NumberCellProperty(
                  gridCell);
              String decimalStr = String.valueOf(buildDecimal(ncp, -3)) + "‰";
              NumberFormat numf = 
                NumberFormat.getInstance();
              numf.setGroupingUsed(true);
              RichTextString richTextString1 = getRichTextString(
                  numf
                  .format(value * 1000.0D), this.workBook);
              if (richTextString1 == null) {
                cell.setCellValue(richTextString1 + decimalStr);
                break;
              } 
              cell.setCellValue(decimalStr);
            } 
          } 
          break;
        case 5:
          date = gridCell.getDateTime();
          if (date != null)
            cell.setCellValue(date); 
          break;
        case 3:
          cellData = gridCell.getCellData();
          if (cellData != null && !cellData.equals("")) {
            if ("|".equals(cellData)) {
              String str = gridCell.getShowText();
              RichTextString richTextString1 = getRichTextString(str, this.workBook);
              if (richTextString1 != null)
                cell.setCellValue(richTextString1); 
              break;
            } 
            double value = Double.parseDouble(cellData);
            cell.setCellValue(value);
          } 
          break;
        case 9:
          linkInfoStr = gridCell.getCellData();
          if (linkInfoStr != null && !"".equals(linkInfoStr)) {
            String[] info = GridCell.parserLinkInformation(linkInfoStr);
            RichTextString hssfStr = getRichTextString(info[0], this.workBook);
            if (hssfStr != null)
              cell.setCellValue(hssfStr); 
          } 
          break;
      } 
    } catch (Exception e) {
      e.printStackTrace();
    } 
  }
  
  private String makeCurFotmat(CurrencyCellProperty curCellProperty) {
    String signStr, nameStr, titleStr, currencyFormat = "#,##0";
    String colorStr = "";
    boolean isBracketNegative = false;
    currencyFormat = String.valueOf(currencyFormat) + buildDecimal((NumberCellProperty)curCellProperty, 0);
    int idx = curCellProperty.getUnitIndex();
    if (curCellProperty.getWarningNegative())
      colorStr = "[Red]"; 
    isBracketNegative = curCellProperty.getBracketNegative();
    switch (curCellProperty.getUnitShowType()) {
      case 0:
        signStr = CurrencyCellProperty.CURRENCY_UNIT_SIGNS[idx];
        if (signStr.length() == 1) {
          currencyFormat = isBracketNegative ? (String.valueOf(signStr) + currencyFormat + 
            ";" + colorStr + signStr + "(" + currencyFormat + ")") : (
            String.valueOf(signStr) + currencyFormat + ";" + colorStr + signStr + 
            "-" + currencyFormat);
          break;
        } 
        if (signStr.length() > 1)
          currencyFormat = isBracketNegative ? ("[$" + signStr + "]" + 
            currencyFormat + ";" + colorStr + "[$" + signStr + 
            "](" + currencyFormat + ")") : ("[$" + signStr + "]" + 
            currencyFormat + ";" + colorStr + "[$" + signStr + 
            "]-" + currencyFormat); 
        break;
      case 1:
        nameStr = CurrencyCellProperty.CURRENCY_UNIT_NAMES[idx];
        currencyFormat = isBracketNegative ? ("[$" + nameStr + "] " + 
          currencyFormat + ";" + colorStr + "[$" + nameStr + "] (" + 
          currencyFormat + ")") : ("[$" + nameStr + "] " + 
          currencyFormat + ";" + colorStr + "[$" + nameStr + "] -" + 
          currencyFormat);
        break;
      case 2:
        nameStr = CurrencyCellProperty.CURRENCY_UNIT_NAMES[idx];
        currencyFormat = isBracketNegative ? (String.valueOf(currencyFormat) + " [$" + 
          nameStr + "];" + colorStr + "(" + currencyFormat + ") [$" + 
          nameStr + "]") : (String.valueOf(currencyFormat) + " [$" + nameStr + "];" + 
          colorStr + "-" + currencyFormat + " [$" + nameStr + "]");
        break;
      case 3:
        titleStr = CurrencyCellProperty.CURRENCY_UNIT_TITLES[idx];
        currencyFormat = isBracketNegative ? ("[$" + titleStr + "] " + 
          currencyFormat + ";" + colorStr + "[$" + titleStr + "] (" + 
          currencyFormat + ")") : ("[$" + titleStr + "] " + 
          currencyFormat + ";" + colorStr + "[$" + titleStr + "] -" + 
          currencyFormat);
        break;
    } 
    return currencyFormat;
  }
  
  private String buildDecimal(NumberCellProperty numberCellProperty, int offset) {
    int decimalCount = numberCellProperty.getDecimal() + offset;
    if (decimalCount > 0) {
      String decimal = ".";
      for (int i = 0; i < decimalCount; i++)
        decimal = String.valueOf(decimal) + "0"; 
      return decimal;
    } 
    return "";
  }
  
  private Font getCellFont(GridCell gridCell) {
    String fontKey = getCellFontKey(gridCell);
    Font font = (Font)this.HSSFFontMap.get(fontKey);
    if (font == null) {
      font = this.workBook.createFont();
      this.HSSFFontMap.put(fontKey, font);
      if (gridCell.getFontBold())
        font.setBoldweight((short)700); 
      HSSFColor hSSFColor = HSSFHelper.getApproximateTextColor(gridCell
          .getFontColor());
      if (hSSFColor != null);
      font.setFontHeight((short)gridCell.getFontHeight());
      font.setItalic(gridCell.getFontItalic());
      font.setFontName(gridCell.getFontName());
      font.setFontHeightInPoints((short)gridCell.getFontSize());
      font.setStrikeout(gridCell.getFontStrikeOut());
      if (gridCell.getFontUnderLine())
        font.setUnderline((byte)1); 
    } 
    return font;
  }
  
  private String getCellFontKey(GridCell cell) {
    StringBuffer key = new StringBuffer();
    key.append(cell.getFontName());
    key.append(cell.getFontSize());
    key.append(cell.getFontColor());
    key.append(cell.getFontHeight());
    key.append(cell.getFontBold());
    key.append(cell.getFontItalic());
    key.append(cell.getFontStrikeOut());
    key.append(cell.getFontUnderLine());
    return key.toString();
  }
  
  private CellStyle buildCellStyle(GridCell gridCell) {
    CellStyle cellStyle = (CellStyle)this.cellStyleHashMap
      .get(gridCell);
    if (cellStyle != null)
      return cellStyle; 
    GridCell tempGridCell = this.gridData.getCellEx(gridCell.getColNum(), 
        gridCell.getRowNum());
    cellStyle = this.workBook.createCellStyle();
    short backColorIdx = 9;
    if (tempGridCell.getSilverHead()) {
      backColorIdx = 22;
    } else if (tempGridCell.getBackStyle() != 0) {
      HSSFColor backColor = HSSFHelper.getApproximateColor(tempGridCell
          .getBackColor());
      if (backColor != null)
        backColorIdx = backColor.getIndex(); 
    } 
    cellStyle.setFillForegroundColor(backColorIdx);
    if (backColorIdx != 58)
      cellStyle.setFillPattern((short)1); 
    short leftBorder = transBorderStyle(tempGridCell.getLEdgeStyle());
    short topBorder = transBorderStyle(tempGridCell.getTEdgeStyle());
    short rightBorder = transBorderStyle(tempGridCell.getREdgeStyle());
    short bottomBorder = transBorderStyle(tempGridCell.getBEdgeStyle());
    HSSFColor leftBorderColor = 
      HSSFHelper.getApproximateEdgeColor(tempGridCell.getLEdgeColor());
    HSSFColor topBorderColor = 
      HSSFHelper.getApproximateEdgeColor(tempGridCell.getTEdgeColor());
    HSSFColor righttBorderColor = 
      HSSFHelper.getApproximateEdgeColor(tempGridCell.getREdgeColor());
    HSSFColor bottomBorderColor = 
      HSSFHelper.getApproximateEdgeColor(tempGridCell.getBEdgeColor());
    cellStyle.setBorderLeft(leftBorder);
    cellStyle.setBorderTop(topBorder);
    cellStyle.setBorderRight(rightBorder);
    cellStyle.setBorderBottom(bottomBorder);
    if (leftBorderColor != null)
      cellStyle.setLeftBorderColor(leftBorderColor.getIndex()); 
    if (topBorderColor != null)
      cellStyle.setTopBorderColor(topBorderColor.getIndex()); 
    if (righttBorderColor != null)
      cellStyle.setRightBorderColor(righttBorderColor.getIndex()); 
    if (bottomBorderColor != null)
      cellStyle.setBottomBorderColor(bottomBorderColor.getIndex()); 
    short horzAlign = transCellAlign(tempGridCell.getHorzAlign(), true);
    short vertAlign = transCellAlign(tempGridCell.getVertAlign(), false);
    cellStyle.setAlignment(horzAlign);
    cellStyle.setVerticalAlignment(vertAlign);
    cellStyle.setWrapText(tempGridCell.getWrapLine());
    cellStyle.setIndention((short)tempGridCell.getIndent());
    Font font = getCellFont(tempGridCell);
    cellStyle.setFont(font);
    if (tempGridCell.getDataType() == 1) {
      short formatIndex = this.workBook.createDataFormat().getFormat(
          "@");
      cellStyle.setDataFormat(formatIndex);
    } 
    if (tempGridCell.getDataType() == 2 && (
      this.doNotCheckNumberIsNull || tempGridCell.getShowText() != null))
      if (tempGridCell.getShowText() == null) {
        NumberCellProperty ncp = new NumberCellProperty(tempGridCell);
        String formatStr = "0";
        if (ncp.getThoundsMark()) {
          formatStr = "#,##0" + 
            buildDecimal(ncp, 0);
        } else {
          formatStr = String.valueOf(formatStr) + buildDecimal(ncp, 0);
        } 
        short formatIndex = this.workBook.createDataFormat().getFormat(
            String.valueOf(formatStr) + " ");
        cellStyle.setDataFormat(formatIndex);
      } else if (tempGridCell.getShowText().endsWith("%")) {
        NumberCellProperty ncp = new NumberCellProperty(tempGridCell);
        String formatStr = "0" + buildDecimal(ncp, -2) + "%";
        short formatIndex = this.workBook.createDataFormat().getFormat(
            formatStr);
        cellStyle.setDataFormat(formatIndex);
      } else if (tempGridCell.getShowText().endsWith("‰")) {
        NumberCellProperty ncp = new NumberCellProperty(gridCell);
        String numStr = "0";
        String colorStr = "";
        if (ncp.getThoundsMark()) {
          numStr = "#,##0" + 
            buildDecimal(ncp, 0);
        } else {
          numStr = String.valueOf(numStr) + buildDecimal(ncp, 0);
        } 
        if (ncp.getWarningNegative())
          colorStr = "[Red]"; 
        String formatStr = "";
        if (ncp.getBracketNegative()) {
          formatStr = String.valueOf(numStr) + ";" + colorStr + "(" + numStr + ")";
        } else {
          formatStr = String.valueOf(numStr) + ";" + colorStr + "-" + numStr;
        } 
        short formatIndex = this.workBook.createDataFormat().getFormat(
            formatStr);
        cellStyle.setDataFormat(formatIndex);
      } else {
        NumberCellProperty ncp = new NumberCellProperty(tempGridCell);
        String formatStr = "0";
        if (ncp.getThoundsMark()) {
          formatStr = "#,##0" + 
            buildDecimal(ncp, 0);
        } else {
          formatStr = String.valueOf(formatStr) + buildDecimal(ncp, 0);
        } 
        short formatIndex = this.workBook.createDataFormat().getFormat(
            String.valueOf(formatStr) + " ");
        cellStyle.setDataFormat(formatIndex);
      }  
    if (tempGridCell.getDataType() == 3) {
      CurrencyCellProperty curCellProperty = new CurrencyCellProperty(
          tempGridCell);
      String currencyFormat = makeCurFotmat(curCellProperty);
      short formatIndex = this.workBook.createDataFormat().getFormat(
          currencyFormat);
      cellStyle.setDataFormat(formatIndex);
    } 
    if (gridCell.getDataType() == 5) {
      short formatIndex = this.workBook.createDataFormat().getFormat(
          "yyyy-m-d");
      cellStyle.setDataFormat(formatIndex);
    } 
    gridCell.getDataType();
    this.cellStyleHashMap.put(tempGridCell, cellStyle);
    return cellStyle;
  }
  
  private short transCellAlign(int align, boolean isHorizontal) {
    if (isHorizontal) {
      switch (align) {
        case 0:
          return 0;
        case 1:
          return 1;
        case 2:
          return 3;
        case 3:
          return 2;
        case 4:
        case 5:
          return 5;
        case 6:
          return 3;
      } 
      return 0;
    } 
    switch (align) {
      case 0:
        return 1;
      case 1:
        return 0;
      case 2:
        return 2;
      case 3:
      case 4:
      case 5:
        return 3;
      case 6:
        return 2;
    } 
    return 1;
  }
  
  private short transBorderStyle(int idx) {
    switch (idx) {
      case 0:
        return 1;
      case 1:
        return 0;
      case 2:
        return 1;
      case 3:
        return 3;
      case 4:
        return 7;
      case 5:
        return 9;
      case 6:
        return 11;
      case 7:
      case 8:
      case 9:
        return 5;
      case 10:
      case 11:
      case 12:
        return 6;
      case 13:
      case 14:
      case 15:
        return 5;
    } 
    return 1;
  }
  
  private void mergeCells() {
    GridFieldList fieldList = this.gridData.merges();
    for (int i = 0; i < fieldList.count(); i++) {
      CellField field = fieldList.get(i);
      int rowFrom = field.top - 1;
      int rowTo = field.bottom - 1;
      int colFrom = field.left - 1;
      int colTo = field.right - 1;
      if (this.addTitle) {
        rowFrom++;
        rowTo++;
      } 
      int sheetFrom = getSheetIndex(colFrom);
      int sheetTo = getSheetIndex(colTo);
      for (int j = sheetFrom; j <= sheetTo; j++) {
        int curColFrom = (j == sheetFrom) ? (colFrom - j * MAX_COLUMN) : 
          0;
        int curColTo = (j == sheetTo) ? (colTo - j * MAX_COLUMN) : 
          MAX_COLUMN;
        getSheet(j).addMergedRegion(
            (CellRangeAddress)new CellRangeAddress(rowFrom, rowTo, curColFrom, 
              curColTo));
      } 
    } 
  }
  
  private void adjustHW() {
    int colCount = this.gridData.getColCount();
    int rowCount = this.gridData.getRowCount();
    if (this.autoAdjust) {
      for (int col = 0; col < colCount; col++)
        getSheet(getSheetIndex(col))
          .autoSizeColumn((short)getCol(col)); 
    } else {
      for (int col = 1; col < colCount; col++) {
        int colWidth = this.gridData.getColWidths(col);
        getSheet(getSheetIndex(col)).setColumnWidth(getCol(col) - 1, 
            HSSFHelper.pixelToWidth(colWidth));
      } 
      for (int row = 1; row < rowCount; row++) {
        if (this.gridData.getRowVisible(row))
          for (int i = 0; i < this.sheets.size(); i++) {
            int rowHeight = this.gridData.getRowHeights(row);
            Row hssfRow = this.addTitle ? getSheet(i).getRow(row) : 
              getSheet(i).getRow(row - 1);
            if (hssfRow != null)
              hssfRow.setHeight(
                  HSSFHelper.pixelToHeight(rowHeight)); 
          }  
      } 
    } 
  }
  
  private void hideRowCol() {
    for (int col = 1; col < this.gridData.getColCount(); col++) {
      if (!this.gridData.getColVisible(col))
        getSheet(getSheetIndex(col)).setColumnHidden(getCol(col) - 1, 
            true); 
    } 
    for (int row = 1; row < this.gridData.getRowCount(); row++) {
      if (!this.gridData.getRowVisible(row))
        for (int i = 0; i < this.sheets.size(); i++) {
          Row hssfRow = this.addTitle ? getSheet(i).getRow(row) : 
            getSheet(i).getRow(row - 1);
          hssfRow.setZeroHeight(true);
        }  
    } 
  }
  
  public void setHSSFFontMap(Map hSSFFontMap) {
    this.HSSFFontMap = hSSFFontMap;
  }
  
  public Map getHSSFFontMap() {
    return this.HSSFFontMap;
  }
  
  private RichTextString getRichTextString(String title, Workbook wb) {
    XSSFRichTextString xSSFRichTextString;
    RichTextString rt = null;
    if (wb instanceof HSSFWorkbook) {
      HSSFRichTextString hSSFRichTextString = new HSSFRichTextString(title);
    } else if (wb instanceof org.apache.poi.xssf.usermodel.XSSFWorkbook) {
      xSSFRichTextString = new XSSFRichTextString(title);
    } else if (wb instanceof org.apache.poi.xssf.streaming.SXSSFWorkbook) {
      xSSFRichTextString = new XSSFRichTextString(title);
    } 
    return (RichTextString)xSSFRichTextString;
  }
}
